# Copyright (c) CSIRO Data61
# Distributed under the terms of the Modified BSD License.
#
# Derived from
# https://github.com/jupyter/docker-stacks/blob/master/base-notebook/jupyter_notebook_config.py
# but adds a default HTML landing page to the notebook.

from jupyter_core.paths import jupyter_data_dir
import subprocess
import os
import errno
import stat

c = get_config()
c.NotebookApp.ip = '*'
c.NotebookApp.port = 8888

# Generate a self-signed certificate
if 'GEN_CERT' in os.environ:
    dir_name = jupyter_data_dir()
    pem_file = os.path.join(dir_name, 'notebook.pem')
    try:
        os.makedirs(dir_name)
    except OSError as exc:  # Python >2.5
        if exc.errno == errno.EEXIST and os.path.isdir(dir_name):
            pass
        else:
            raise
    # Generate a certificate if one doesn't exist on disk
    subprocess.check_call(['openssl', 'req', '-new',
                           '-newkey', 'rsa:2048',
                           '-days', '365',
                           '-nodes', '-x509',
                           '-subj', '/C=XX/ST=XX/L=XX/O=generated/CN=generated',
                           '-keyout', pem_file,
                           '-out', pem_file])
    # Restrict access to the file
    os.chmod(pem_file, stat.S_IRUSR | stat.S_IWUSR)
    c.NotebookApp.certfile = pem_file

# Start with a default landing page if 'DEFAULT_LANDING' is set.
if 'DEFAULT_LANDING' in os.environ:
    c.NotebookApp.default_url = os.environ.get('DEFAULT_LANDING')

c.NotebookApp.allow_origin = '*'
